/*
 * Copyright 2007-2008 WorldWide Conferencing, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 */
package net.liftweb.util

import org.specs._
import org.specs.runner._
import org.specs.util.WaitFor
import scala.actors.Actor
import java.util.concurrent._

class ActorPingUnitTest extends JUnit3(ActorPingUnit)
object ActorPingUnitRunner extends ConsoleRunner(ActorPingUnit)
object ActorPingUnit extends Specification with PingedService with WaitFor {
  def pingService = {
    service.start
    ActorPing.schedule(service, Alive, 10)
    waitFor(100.ms)
    service.pinged must beTrue
  }
  "The ActorPing object" can {
    "be restarted twice" in {
      ActorPing.restart
      ActorPing.restart
      pingService
    }
    "be shutdown and restarted" in {
      ActorPing.shutdown
      ActorPing.restart
      pingService
    }
    "be shutdown twice" in {
      ActorPing.shutdown
      ActorPing.shutdown
      pingService must throwAn(ActorPingException("", null))
    }
  }
}
