package net.liftweb.util
import org.specs._
import scala.util.parsing.input._  
import org.specs.runner._
import scala.util.parsing.combinator.Parsers
import org.scalacheck._
import Gen._
import Prop._
import org.specs.matcher.ScalacheckParameters._

class JSONSpecTest extends JUnit3(JSONSpec)
object JSONRunner extends ConsoleRunner(JSONSpec)
object JSONSpec extends Specification {
  "The JSON Parser" should {
    "parse very long strings without causing a stack overflow" in {
      val longJson = """{"command":"test","params":{"participants":"","subject":"You have been invited to play the Innovation Game: Buy a Feature","reply-to":"no-reply@innovationgames.com","body":"Dear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\nDear %name%,\r\n\r\nYou have been invited to play the Innovation Game(r) Buy a Feature to help us better understand what you're looking for in future products and services.\r\n\r\nTo play the game, click on the link below. This link will open a web site that will present you with a list of features. Each feature costs a certain amount of money. You will be given some money to spend on the features that you want to purchase. Since you probably won't have enough money to purchase everything you want, we encourage you to collaborate with other players to purchase features that you can't afford.\r\n\r\nHere are the details of the game.\r\n\r\nDate: %date%\r\n\r\nTime: %time%\r\n\r\nClick on this link to get see all of the features that are in this game: %featurelink%\r\n\r\nClick on this link to play the game: %playlink%\r\n\r\nHere are some additional tips for playing the game:\r\n\r\n* Your should enable pop-ups and cookies for the games.\r\n\r\nSincerely,\r\n\r\n%sendername%\r\n"}}"""
      
      val parsed = JSONParser.parse(longJson)
      parsed.isDefined must be(true)
      
      // "reply-to"
      (parsed match {
        case Full(m: Map[String, Any]) =>
        (m.get("command"), m.get("params")) match {
          case (Some(c: String), Some(m2: Map[String, String])) =>
          (c, m2("participants"), m2("reply-to"))

          case _ => ("", "", "")
        }
        
        case _ => ("", "", "")
      }) must be_== ("test", "", "no-reply@innovationgames.com")
    }
  }
}
 